/* -*-c-*-
 *
 * Key loading and storing
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Trivial IP Encryption (TrIPE).
 *
 * TrIPE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TrIPE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TrIPE; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Header files ------------------------------------------------------*/

#include "tripe.h"

/*----- Key groups --------------------------------------------------------*/

/* The key-loading functions here must fill in the kdata slot @g@ and
 * either @kpriv@ or @kpub@ as appropriate.  The caller will take care of
 * determining @kpub@ given a private key, and of ensuring that @kpriv@ is
 * null for a public key.
 */

typedef struct kgops {
  const char *ty;
  int (*loadpriv)(key_data *, kdata *, dstr *, dstr *);
  int (*loadpub)(key_data *, kdata *, dstr *, dstr *);
} kgops;

/* --- Diffie-Hellman --- */

static int kgdh_priv(key_data *d, kdata *kd, dstr *t, dstr *e)
{
  key_packstruct kps[DH_PRIVFETCHSZ];
  key_packdef *kp;
  dh_priv dp;
  int rc;

  kp = key_fetchinit(dh_privfetch, kps, &dp);
  if ((rc = key_unpack(kp, d, t)) != 0) {
    a_format(e, "unpack-failed", "%s", key_strerror(rc), A_END);
    goto fail_0;
  }
  kd->g = group_prime(&dp.dp);
  kd->kpriv = MP_COPY(dp.x);
  rc = 0;
  goto done;
fail_0:
  rc = -1;
done:
  key_fetchdone(kp);
  return (rc);
}

static int kgdh_pub(key_data *d, kdata *kd, dstr *t, dstr *e)
{
  key_packstruct kps[DH_PUBFETCHSZ];
  key_packdef *kp;
  dh_pub dp;
  int rc;

  kp = key_fetchinit(dh_pubfetch, kps, &dp);
  if ((rc = key_unpack(kp, d, t)) != 0) {
    a_format(e, "unpack-failed", "%s", key_strerror(rc), A_END);
    goto fail_0;
  }
  kd->g = group_prime(&dp.dp);
  kd->kpub = G_CREATE(kd->g);
  if (G_FROMINT(kd->g, kd->kpub, dp.y)) {
    a_format(e, "bad-public-vector", A_END);
    goto fail_1;
  }
  rc = 0;
  goto done;
fail_1:
  G_DESTROY(kd->g, kd->kpub);
  G_DESTROYGROUP(kd->g);
fail_0:
  rc = -1;
done:
  key_fetchdone(kp);
  return (rc);
}

static const kgops kgdh_ops = { "dh", kgdh_priv, kgdh_pub };

/* --- Elliptic curve --- */

static int kgec_priv(key_data *d, kdata *kd, dstr *t, dstr *e)
{
  key_packstruct kps[EC_PRIVFETCHSZ];
  key_packdef *kp;
  ec_priv ep;
  ec_info ei;
  const char *err;
  int rc;

  kp = key_fetchinit(ec_privfetch, kps, &ep);
  if ((rc = key_unpack(kp, d, t)) != 0) {
    a_format(e, "unpack-failed", "%s", key_strerror(rc), A_END);
    goto fail_0;
  }
  if ((err = ec_getinfo(&ei, ep.cstr)) != 0) {
    a_format(e, "decode-failed", "%s", err, A_END);
    goto fail_0;
  }
  kd->g = group_ec(&ei);
  kd->kpriv = MP_COPY(ep.x);
  rc = 0;
  goto done;
fail_0:
  rc = -1;
done:
  key_fetchdone(kp);
  return (rc);
}

static int kgec_pub(key_data *d, kdata *kd, dstr *t, dstr *e)
{
  key_packstruct kps[EC_PUBFETCHSZ];
  key_packdef *kp;
  ec_pub ep;
  ec_info ei;
  const char *err;
  int rc;

  kp = key_fetchinit(ec_pubfetch, kps, &ep);
  if ((rc = key_unpack(kp, d, t)) != 0) {
    a_format(e, "unpack-failed", "%s", key_strerror(rc), A_END);
    goto fail_0;
  }
  if ((err = ec_getinfo(&ei, ep.cstr)) != 0) {
    a_format(e, "decode-failed", "%s", err, A_END);
    goto fail_0;
  }
  kd->g = group_ec(&ei);
  kd->kpub = G_CREATE(kd->g);
  if (G_FROMEC(kd->g, kd->kpub, &ep.p)) {
    a_format(e, "bad-public-vector", A_END);
    goto fail_1;
  }
  rc = 0;
  goto done;
fail_1:
  G_DESTROY(kd->g, kd->kpub);
  G_DESTROYGROUP(kd->g);
fail_0:
  rc = -1;
done:
  key_fetchdone(kp);
  return (rc);
}

static const kgops kgec_ops = { "ec", kgec_priv, kgec_pub };

/* --- Table of supported key types --- */

static const kgops *kgtab[] = { &kgdh_ops, &kgec_ops, 0 };

/*----- Algswitch stuff ---------------------------------------------------*/

/* --- @algs_get@ --- *
 *
 * Arguments:	@algswitch *a@ = where to put the algorithms
 *		@dstr *e@ = where to write errror tokens
 *		@key_file *kf@ = key file
 *		@key *k@ = key to inspect
 *
 * Returns:	Zero if OK; nonzero on error.
 *
 * Use:		Extracts an algorithm choice from a key.
 */

static int algs_get(algswitch *a, dstr *e, key_file *kf, key *k)
{
  const char *p;
  char *q, *qq;
  dstr d = DSTR_INIT;
  int rc = -1;

  /* --- Symmetric encryption for bulk data --- */

  if ((p = key_getattr(kf, k, "cipher")) == 0) p = "blowfish-cbc";
  if ((a->c = gcipher_byname(p)) == 0) {
    a_format(e, "unknown-cipher", "%s", p, A_END);
    goto done;
  }

  /* --- Hash function --- */

  if ((p = key_getattr(kf, k, "hash")) == 0) p = "rmd160";
  if ((a->h = ghash_byname(p)) == 0) {
    a_format(e, "unknown-hash", "%s", p, A_END);
    goto done;
  }

  /* --- Symmetric encryption for key derivation --- */

  if ((p = key_getattr(kf, k, "mgf")) == 0) {
    dstr_reset(&d);
    dstr_putf(&d, "%s-mgf", a->h->name);
    p = d.buf;
  }
  if ((a->mgf = gcipher_byname(p)) == 0) {
    a_format(e, "unknown-mgf-cipher", "%s", p, A_END);
    goto done;
  }

  /* --- Message authentication for bulk data --- */

  if ((p = key_getattr(kf, k, "mac")) != 0) {
    dstr_reset(&d);
    dstr_puts(&d, p);
    if ((q = strchr(d.buf, '/')) != 0)
      *q++ = 0;
    if ((a->m = gmac_byname(d.buf)) == 0) {
      a_format(e, "unknown-mac", "%s", d.buf, A_END);
      goto done;
    }
    if (!q)
      a->tagsz = a->m->hashsz;
    else {
      unsigned long n = strtoul(q, &qq, 0);
      if (*qq)  {
	a_format(e, "bad-tag-length-string", "%s", q, A_END);
	goto done;
      }
      if (n%8 || n/8 > a->m->hashsz) {
	a_format(e, "bad-tag-length", "%lu", n, A_END);
	goto done;
      }
      a->tagsz = n/8;
    }
  } else {
    dstr_reset(&d);
    dstr_putf(&d, "%s-hmac", a->h->name);
    if ((a->m = gmac_byname(d.buf)) == 0) {
      a_format(e, "no-hmac-for-hash", "%s", a->h->name, A_END);
      goto done;
    }
    a->tagsz = a->h->hashsz/2;
  }

  rc = 0;
done:
  dstr_destroy(&d);
  return (rc);
}

/* --- @algs_check@ --- *
 *
 * Arguments:	@algswitch *a@ = a choice of algorithms
 *		@dstr *e@ = where to write error tokens
 *		@const group *g@ = the group we're working in
 *
 * Returns:	Zero if OK; nonzero on error.
 *
 * Use:		Checks an algorithm choice for sensibleness.  This also
 *		derives some useful information from the choices, and you
 *		must call this before committing the algorithm selection
 *		for use by @keyset@ functions.
 */

static int algs_check(algswitch *a, dstr *e, const group *g)
{
  /* --- Derive the key sizes --- *
   *
   * Must ensure that we have non-empty keys.  This isn't ideal, but it
   * provides a handy sanity check.  Also must be based on a 64- or 128-bit
   * block cipher or we can't do the data expiry properly.
   */

  a->hashsz = a->h->hashsz;
  if ((a->cksz = keysz(a->hashsz, a->c->keysz)) == 0) {
    a_format(e, "cipher", "%s", a->c->name,
	     "no-key-size", "%lu", (unsigned long)a->hashsz,
	     A_END);
    return (-1);
  }
  if ((a->mksz = keysz(a->hashsz, a->m->keysz)) == 0) {
    a_format(e, "mac", "%s", a->m->name,
	     "no-key-size", "%lu", (unsigned long)a->hashsz,
	     A_END);
    return (-1);
  }

  /* --- Derive the data limit --- */

  if (a->c->blksz < 16) a->expsz = MEG(64);
  else a->expsz = MEG(2048);

  /* --- Ensure the MGF accepts hashes as keys --- */

  if (keysz(a->hashsz, a->mgf->keysz) != a->hashsz) {
    a_format(e, "mgf", "%s", a->mgf->name,
	     "restrictive-key-schedule",
	     A_END);
    return (-1);
  }

  /* --- All ship-shape and Bristol-fashion --- */

  return (0);
}

/* --- @km_samealgsp@ --- *
 *
 * Arguments:	@const kdata *kdx, *kdy@ = two key data objects
 *
 * Returns:	Nonzero if their two algorithm selections are the same.
 *
 * Use:		Checks sameness of algorithm selections: used to ensure that
 *		peers are using sensible algorithms.
 */

int km_samealgsp(const kdata *kdx, const kdata *kdy)
{
  const algswitch *a = &kdx->algs, *aa = &kdy->algs;

  return (group_samep(kdx->g, kdy->g) && a->c == aa->c &&
	  a->mgf == aa->mgf && a->h == aa->h &&
	  a->m == aa->m && a->tagsz == aa->tagsz);
}

/*----- Key data and key nodes --------------------------------------------*/

typedef struct keyhalf {
  const char *kind;
  int (*load)(const kgops *, key_data *, kdata *, dstr *, dstr *);
  const char *kr;
  key_file *kf;
  fwatch w;
  sym_table tab;
} keyhalf;

/* --- @kh_loadpub@, @kh_loadpriv@ --- *
 *
 * Arguments:	@const kgops *ko@ = key-group operations for key type
 *		@key_data *d@ = key data object as stored in keyring
 *		@kdata *kd@ = our key-data object to fill in
 *		@dstr *t@ = the key tag name
 *		@dstr *e@ = a string to write error tokens to
 *
 * Returns:	Zero on success, @-1@ on error.
 *
 * Use:		These functions handle the main difference between public and
 *		private key halves.  They are responsible for setting @g@,
 *		@kpriv@ and @kpub@ appropriately in all keys, handling the
 *		mismatch between the largely half-indifferent calling code
 *		and the group-specific loading functions.
 *
 *		The function @kh_loadpriv@ is also responsible for checking
 *		the group for goodness.  We don't bother checking public
 *		keys, because each public key we actually end up using must
 *		share a group with a private key which we'll already have
 *		checked.
 */

static int kh_loadpub(const kgops *ko, key_data *d, kdata *kd,
		      dstr *t, dstr *e)
{
  int rc;

  if ((rc = ko->loadpub(d, kd, t, e)) != 0)
    goto fail_0;
  if (group_check(kd->g, kd->kpub)) {
    a_format(e, "bad-public-group-element");
    goto fail_1;
  }
  kd->kpriv = 0;
  return (0);

fail_1:
  G_DESTROY(kd->g, kd->kpub);
  G_DESTROYGROUP(kd->g);
fail_0:
  return (-1);
}

static int kh_loadpriv(const kgops *ko, key_data *d, kdata *kd,
		       dstr *t, dstr *e)
{
  int rc;
  const char *err;

  if ((rc = ko->loadpriv(d, kd, t, e)) != 0)
    goto fail_0;
  if ((err = G_CHECK(kd->g, &rand_global)) != 0) {
    a_format(e, "bad-group", "%s", err, A_END);
    goto fail_1;
  }
  kd->kpub = G_CREATE(kd->g);
  G_EXP(kd->g, kd->kpub, kd->g->g, kd->kpriv);
  return (0);

fail_1:
  mp_drop(kd->kpriv);
  G_DESTROYGROUP(kd->g);
fail_0:
  return (-1);
}

static struct keyhalf
  priv = { "private", kh_loadpriv },
  pub = { "public", kh_loadpub };

/* --- @keymoan@ --- *
 *
 * Arguments:	@const char *file@ = name of the file
 *		@int line@ = line number in file
 *		@const char *msg@ = error message
 *		@void *p@ = argument pointer (indicates which keyring)
 *
 * Returns:	---
 *
 * Use:		Reports an error message about loading a key file.
 */

static void keymoan(const char *file, int line, const char *msg, void *p)
{
  keyhalf *kh = p;

  if (!line) {
    a_warn("KEYMGMT", "%s-keyring", kh->kind, "%s", file,
	   "io-error", "?ERRNO", A_END);
  } else {
    a_warn("KEYMGMT", "%s-keyring", kh->kind, "%s", file, "line", "%d", line,
	   "%s", msg, A_END);
  }
}

/* --- @kh_reopen@ --- *
 *
 * Arguments:	@keyhalf *kh@ = pointer to keyhalf structure
 *
 * Returns:	Zero on success, @-1@ on error.
 *
 * Use:		Reopens the key file for the appropriate key half.  If this
 *		fails, everything is left as it was; if it succeeds, then the
 *		old file is closed (if it was non-null) and the new one put
 *		in its place.
 */

static int kh_reopen(keyhalf *kh)
{
  key_file *kf = CREATE(key_file);

  if (key_open(kf, kh->kr, KOPEN_READ, keymoan, kh)) {
    a_warn("KEYMGMT", "%s-keyring", kh->kind, "%s", kh->kr,
	   "io-error", "?ERRNO", A_END);
    DESTROY(kf);
    return (-1);
  } else {
    if (kh->kf) {
      key_close(kh->kf);
      DESTROY(kh->kf);
    }
    kh->kf = kf;
    return (0);
  }
}

/* --- @kh_init@ --- *
 *
 * Arguments:	@keyhalf *kh@ = pointer to keyhalf structure to set up
 *		@const char *kr@ = name of the keyring file
 *
 * Returns:	---
 *
 * Use:		Initialize a keyhalf structure, maintaining the private or
 *		public keys.  Intended to be called during initialization:
 *		exits if there's some kind of problem.
 */

static void kh_init(keyhalf *kh, const char *kr)
{
  kh->kr = kr;
  fwatch_init(&kh->w, kr);
  sym_create(&kh->tab);
  kh->kf = 0;

  if (kh_reopen(kh))
    die(EXIT_FAILURE, "failed to load %s keyring `%s'", kh->kind, kr);
}

/* --- @kh_load@ --- *
 *
 * Arguments:	@keyhalf *kh@ = pointer to keyhalf
 *		@const char *tag@ = key tag to be loaded
 *		@int complainp@ = whether to complain about missing keys
 *
 * Returns:	Pointer to a @kdata@ structure if successful, or null on
 *		failure.
 *
 * Use:		Attempts to load a key from the current key file.  This
 *		function always reads data from the file: it's used when
 *		there's a cache miss from @kh_find@, and when refreshing the
 *		known keys in @kh_refresh@.  The returned kdata has a
 *		reference count of exactly 1, and has no home knode.
 */

static kdata *kh_load(keyhalf *kh, const char *tag, int complainp)
{
  dstr t = DSTR_INIT;
  dstr e = DSTR_INIT;
  key *k;
  key_data **d;
  kdata *kd;
  const char *ty;
  const kgops **ko;

  /* --- Find the key and grab its tag --- */

  if (key_qtag(kh->kf, tag, &t, &k, &d)) {
    if (complainp) {
      a_warn("KEYMGMT", "%s-keyring", kh->kind, "%s", kh->kr,
	     "key-not-found", "%s", tag, A_END);
    }
    goto fail_0;
  }

  /* --- Find the key's group type and the appropriate operations --- *
   *
   * There are several places to look for the key type.  The most obvious is
   * the `kx-group' key attribute.  But there's also the key type itself, for
   * compatibility reasons.
   */

  ty = key_getattr(kh->kf, k, "kx-group");
  if (!ty && strncmp(k->type, "tripe-", 6) == 0) ty = k->type + 6;
  if (!ty) ty = "dh";

  for (ko = kgtab; *ko; ko++)
    if (strcmp((*ko)->ty, ty) == 0) goto foundko;
  a_warn("KEYMGMT", "%s-keyring", kh->kind,
	 "%s", kh->kr, "key", "%s", t.buf,
	 "unknown-group-type", "%s", ty, A_END);
  goto fail_0;

foundko:
  kd = CREATE(kdata);
  if (kh->load(*ko, *d, kd, &t, &e)) {
    a_warn("KEYMGMT", "%s-keyring", kh->kind,
	   "%s", kh->kr, "key" "%s", t.buf,
	   "*%s", e.buf, A_END);
    goto fail_1;
  }

  if (algs_get(&kd->algs, &e, kh->kf, k) ||
      (kd->kpriv && algs_check(&kd->algs, &e, kd->g))) {
    a_warn("KEYMGMT", "%s-keyring", kh->kind,
	   "%s", kh->kr, "key", "%s", t.buf,
	   "*%s", e.buf, A_END);
    goto fail_2;
  }

  kd->tag = xstrdup(t.buf);
  kd->indexsz = mp_octets(kd->g->r);
  kd->ref = 1;
  kd->kn = 0;
  kd->t_exp = k->exp;

  IF_TRACING(T_KEYMGMT, {
    trace(T_KEYMGMT, "keymgmt: loaded %s key `%s'", kh->kind, t.buf);
    IF_TRACING(T_CRYPTO, {
      trace(T_CRYPTO, "crypto: r = %s", mpstr(kd->g->r));
      trace(T_CRYPTO, "crypto: h = %s", mpstr(kd->g->h));
      if (kd->kpriv)
	trace(T_CRYPTO, "crypto: x = %s", mpstr(kd->kpriv));
      trace(T_CRYPTO, "crypto: cipher = %s", kd->algs.c->name);
      trace(T_CRYPTO, "crypto: mgf = %s", kd->algs.mgf->name);
      trace(T_CRYPTO, "crypto: hash = %s", kd->algs.h->name);
      trace(T_CRYPTO, "crypto: mac = %s/%lu",
	    kd->algs.m->name, (unsigned long)kd->algs.tagsz * 8);
    })
  })

  goto done;

fail_2:
  if (kd->kpriv) mp_drop(kd->kpriv);
  G_DESTROY(kd->g, kd->kpub);
  G_DESTROYGROUP(kd->g);
fail_1:
  DESTROY(kd);
fail_0:
  kd = 0;
done:
  dstr_destroy(&t);
  dstr_destroy(&e);
  return (kd);
}

/* --- @kh_find@ --- *
 *
 * Arguments:	@keyhalf *kh@ = pointer to the keyhalf
 *		@const char *tag@ = key to be obtained
 *		@int complainp@ = whether to complain about missing keys
 *
 * Returns:	A pointer to the kdata, or null on error.
 *
 * Use:		Obtains kdata, maybe from the cache.  This won't update a
 *		stale cache entry, though @kh_refresh@ ought to have done
 *		that already.  The returned kdata object may be shared with
 *		other users.  (One of this function's responsibilities, over
 *		@kh_load@, is to set the home knode of a freshly loaded
 *		kdata.)
 */

static kdata *kh_find(keyhalf *kh, const char *tag, int complainp)
{
  knode *kn;
  kdata *kd;
  unsigned f;

  kn = sym_find(&kh->tab, tag, -1, sizeof(knode), &f);

  if (f) {
    if (kn->f & KNF_BROKEN) {
      T( if (complainp)
	   trace(T_KEYMGMT, "keymgmt: key `%s' marked as broken", tag); )
      return (0);
    }

    kd = kn->kd;
    if (kd) kd->ref++;
    T( trace(T_KEYMGMT, "keymgmt: %scache hit for key `%s'",
	     kd ? "" : "negative ", tag); )
    return (kd);
  } else {
    kd = kh_load(kh, tag, complainp);
    kn->kd = kd;
    kn->kh = kh;
    kn->f = 0;
    if (!kd)
      kn->f |= KNF_BROKEN;
    else {
      kd->kn = kn;
      kd->ref++;
    }
    return (kd);
  }
}

/* --- @kh_refresh@ --- *
 *
 * Arguments:	@keyhalf *kh@ = pointer to the keyhalf
 *
 * Returns:	Zero if nothing needs to be done; nonzero if peers should
 *		refresh their keys.
 *
 * Use:		Refreshes cached keys from files.
 *
 *		Each active knode is examined to see if a new key is
 *		available: the return value is nonzero if any new keys are.
 *		A key is considered new if its algorithms, public key, or
 *		expiry time are/is different.
 *
 *		Stub knodes (with no kdata attached) are removed, so that a
 *		later retry can succeed if the file has been fixed.  (This
 *		doesn't count as a change, since no peers should be relying
 *		on a nonexistent key.)
 */

static int kh_refresh(keyhalf *kh)
{
  knode *kn;
  kdata *kd;
  sym_iter i;
  int changep = 0;

  if (!fwatch_update(&kh->w, kh->kr) || kh_reopen(kh))
    return (0);

  T( trace(T_KEYMGMT, "keymgmt: rescan %s keyring `%s'", kh->kind, kh->kr); )
  for (sym_mkiter(&i, &kh->tab); (kn = sym_next(&i)) != 0; ) {
    if (!kn->kd) {
      T( trace(T_KEYMGMT, "keymgmt: discard stub entry for key `%s'",
	       SYM_NAME(kn)); )
      sym_remove(&kh->tab, kn);
      continue;
    }
    if ((kd = kh_load(kh, SYM_NAME(kn), 1)) == 0) {
      if (!(kn->f & KNF_BROKEN)) {
	T( trace(T_KEYMGMT, "keymgmt: failed to load new key `%s': "
		 "marking it as broken",
		 SYM_NAME(kn)); )
	kn->f |= KNF_BROKEN;
      }
      continue;
    }
    kn->f &= ~KNF_BROKEN;
    if (kd->t_exp == kn->kd->t_exp &&
	km_samealgsp(kd, kn->kd) &&
	G_EQ(kd->g, kd->kpub, kn->kd->kpub)) {
      T( trace(T_KEYMGMT, "keymgmt: key `%s' unchanged", SYM_NAME(kn)); )
      continue;
    }
    T( trace(T_KEYMGMT, "keymgmt: loaded new version of key `%s'",
	     SYM_NAME(kn)); )
    km_unref(kn->kd);
    kd->kn = kn;
    kn->kd = kd;
    changep = 1;
  }

  return (changep);
}

/*----- Main code ---------------------------------------------------------*/

const char *tag_priv;
kdata *master;

/* --- @km_init@ --- *
 *
 * Arguments:	@const char *privkr@ = private keyring file
 *		@const char *pubkr@ = public keyring file
 *		@const char *ptag@ = default private-key tag
 *
 * Returns:	---
 *
 * Use:		Initializes the key-management machinery, loading the
 *		keyrings and so on.
 */

void km_init(const char *privkr, const char *pubkr, const char *ptag)
{
  const gchash *const *hh;

  for (hh = ghashtab; *hh; hh++) {
    if ((*hh)->hashsz > MAXHASHSZ) {
      die(EXIT_FAILURE, "INTERNAL ERROR: %s hash length %lu > MAXHASHSZ %d",
	  (*hh)->name, (unsigned long)(*hh)->hashsz, MAXHASHSZ);
    }
  }

  kh_init(&priv, privkr);
  kh_init(&pub, pubkr);

  tag_priv = ptag;
  if ((master = km_findpriv(ptag)) == 0) exit(EXIT_FAILURE);
}

/* --- @km_reload@ --- *
 *
 * Arguments:	---
 *
 * Returns:	Zero if OK, nonzero to force reloading of keys.
 *
 * Use:		Checks the keyrings to see if they need reloading.
 */

int km_reload(void)
{
  int changep = 0;
  kdata *kd;

  if (kh_refresh(&priv)) {
    changep = 1;
    kd = master->kn->kd;
    if (kd != master) {
      km_unref(master);
      km_ref(kd);
      master = kd;
    }
  }
  if (kh_refresh(&pub))
    changep = 1;
  return (changep);
}

/* --- @km_findpub@, @km_findpriv@ --- *
 *
 * Arguments:	@const char *tag@ = key tag to load
 *
 * Returns:	Pointer to the kdata object if successful, or null on error.
 *
 * Use:		Fetches a public or private key from the keyring.
 */

kdata *km_findpub(const char *tag) { return (kh_find(&pub, tag, 1)); }

kdata *km_findpriv(const char *tag)
{
  kdata *kd;

  /* Unpleasantness for the sake of compatibility. */
  if (!tag && (kd = kh_find(&priv, "tripe", 0)) != 0) return (kd);
  else return (kh_find(&priv, tag ? tag : "tripe-dh", 1));
}

/* --- @km_tag@ --- *
 *
 * Arguments:	@kdata *kd@ - pointer to the kdata object
 *
 * Returns:	A pointer to the short tag by which the kdata was loaded.
 */

const char *km_tag(kdata *kd) { return (SYM_NAME(kd->kn)); }

/* --- @km_ref@ --- *
 *
 * Arguments:	@kdata *kd@ = pointer to the kdata object
 *
 * Returns:	---
 *
 * Use:		Claim a new reference to a kdata object.
 */

void km_ref(kdata *kd) { kd->ref++; }

/* --- @km_unref@ --- *
 *
 * Arguments:	@kdata *kd@ = pointer to the kdata object
 *
 * Returns:	---
 *
 * Use:		Releases a reference to a kdata object.
 */

void km_unref(kdata *kd)
{
  if (--kd->ref) return;
  if (kd->kpriv) mp_drop(kd->kpriv);
  G_DESTROY(kd->g, kd->kpub);
  xfree(kd->tag);
  G_DESTROYGROUP(kd->g);
}

/*----- That's all, folks -------------------------------------------------*/
