/* -*-c-*-
 *
 * Main header file for TrIPE
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Trivial IP Encryption (TrIPE).
 *
 * TrIPE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TrIPE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TrIPE; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef TRIPE_H
#define TRIPE_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include "config.h"

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <pwd.h>
#include <grp.h>

#include <mLib/alloc.h>
#include <mLib/arena.h>
#include <mLib/base64.h>
#include <mLib/bres.h>
#include <mLib/daemonize.h>
#include <mLib/dstr.h>
#include <mLib/env.h>
#include <mLib/fdflags.h>
#include <mLib/fdpass.h>
#include <mLib/fwatch.h>
#include <mLib/hash.h>
#include <mLib/macros.h>
#include <mLib/mdup.h>
#include <mLib/mdwopt.h>
#include <mLib/quis.h>
#include <mLib/report.h>
#include <mLib/sel.h>
#include <mLib/selbuf.h>
#include <mLib/sig.h>
#include <mLib/str.h>
#include <mLib/sub.h>
#include <mLib/trace.h>
#include <mLib/tv.h>
#include <mLib/versioncmp.h>

#include <catacomb/buf.h>

#include <catacomb/gcipher.h>
#include <catacomb/gmac.h>
#include <catacomb/grand.h>
#include <catacomb/key.h>
#include <catacomb/paranoia.h>

#include <catacomb/noise.h>
#include <catacomb/rand.h>

#include <catacomb/mp.h>
#include <catacomb/mprand.h>
#include <catacomb/dh.h>
#include <catacomb/ec.h>
#include <catacomb/ec-keys.h>
#include <catacomb/group.h>

#include "priv.h"
#include "protocol.h"
#include "slip.h"
#include "util.h"

#undef sun

/*----- Magic numbers -----------------------------------------------------*/

/* --- Trace flags --- */

#define T_TUNNEL 1u
#define T_PEER 2u
#define T_PACKET 4u
#define T_ADMIN 8u
#define T_CRYPTO 16u
#define T_KEYSET 32u
#define T_KEYEXCH 64u
#define T_KEYMGMT 128u
#define T_CHAL 256u
/* T_PRIVSEP  in priv.h */

#define T_ALL 1023u

/* --- Units --- */

#define SEC(n) (n##u)
#define MIN(n) (n##u * 60u)
#define F_2P32 (65536.0*65536.0)
#define MEG(n) (n##ul * 1024ul * 1024ul)

/* --- Timing parameters --- */

#define T_EXP MIN(60)			/* Expiry time for a key */
#define T_REGEN MIN(40)			/* Regeneration time for a key */

#define T_VALID SEC(20)			/* Challenge validity period */
#define T_RETRYMIN SEC(2)		/* Minimum retry interval */
#define T_RETRYMAX MIN(5)		/* Maximum retry interval */
#define T_RETRYGROW (5.0/4.0)		/* Retry interval growth factor */

#define T_WOBBLE (1.0/3.0)		/* Relative timer randomness */

/* --- Other things --- */

#define PKBUFSZ 65536

/*----- Cipher selections -------------------------------------------------*/

typedef struct algswitch {
  const gccipher *c;			/* Symmetric encryption scheme */
  const gccipher *mgf;			/* Mask-generation function */
  const gchash *h;			/* Hash function */
  const gcmac *m;			/* Message authentication code */
  size_t hashsz;			/* Hash output size */
  size_t tagsz;				/* Length to truncate MAC tags */
  size_t expsz;				/* Size of data to process */
  size_t cksz, mksz;			/* Key lengths for @c@ and @m@ */
} algswitch;

typedef struct kdata {
  unsigned ref;				/* Reference counter */
  struct knode *kn;			/* Pointer to cache entry */
  char *tag;				/* Full tag name of the key */
  group *g;				/* The group we work in */
  size_t indexsz;			/* Size of exponent for the group */
  mp *kpriv;				/* The private key (or null) */
  ge *kpub;				/* The public key */
  time_t t_exp;				/* Expiry time of the key */
  algswitch algs;			/* Collection of algorithms */
} kdata;

typedef struct knode {
  sym_base _b;				/* Symbol table intrusion */
  unsigned f;				/* Various flags */
#define KNF_BROKEN 1u			/*   Don't use this key any more */
  struct keyhalf *kh;			/* Pointer to the home keyhalf */
  kdata *kd;				/* Pointer to the key data */
} knode;

#define MAXHASHSZ 64			/* Largest possible hash size */

#define HASH_STRING(h, s) GH_HASH((h), (s), sizeof(s))

/*----- Data structures ---------------------------------------------------*/

/* --- Socket addresses --- *
 *
 * A magic union of supported socket addresses.
 */

typedef union addr {
  struct sockaddr sa;
  struct sockaddr_in sin;
} addr;

/* --- Mapping keyed on addresses --- */

typedef struct addrmap {
  hash_table t;
  size_t load;
} addrmap;

typedef struct addrmap_base {
  hash_base b;
  addr a;
} addrmap_base;

/* --- Sequence number checking --- */

typedef struct seqwin {
  uint32 seq;				/* First acceptable input sequence */
  uint32 win;				/* Window of acceptable numbers */
} seqwin;

#define SEQ_WINSZ 32			/* Bits in sequence number window */

/* --- A symmetric keyset --- *
 *
 * A keyset contains a set of symmetric keys for encrypting and decrypting
 * packets.  Keysets are stored in a list, sorted in reverse order of
 * creation, so that the most recent keyset (the one most likely to be used)
 * is first.
 *
 * Each keyset has a time limit and a data limit.  The keyset is destroyed
 * when either it has existed for too long, or it has been used to encrypt
 * too much data.  New key exchanges are triggered when keys are close to
 * expiry.
 */

typedef struct keyset {
  struct keyset *next;			/* Next active keyset in the list */
  unsigned ref;				/* Reference count for keyset */
  struct peer *p;			/* Pointer to peer structure */
  time_t t_exp;				/* Expiry time for this keyset */
  unsigned long sz_exp, sz_regen;	/* Data limits for the keyset */
  T( unsigned seq; )			/* Sequence number for tracing */
  unsigned f;				/* Various useful flags */
  gcipher *cin, *cout;			/* Keyset ciphers for encryption */
  size_t tagsz;				/* Length to truncate MAC tags */
  gmac *min, *mout;			/* Keyset MACs for integrity */
  uint32 oseq;				/* Outbound sequence number */
  seqwin iseq;				/* Inbound sequence number */
} keyset;

#define KSF_LISTEN 1u			/* Don't encrypt packets yet */
#define KSF_LINK 2u			/* Key is in a linked list */

#define KSERR_REGEN -1			/* Regenerate keys */
#define KSERR_NOKEYS -2			/* No keys left */
#define KSERR_DECRYPT -3		/* Unable to decrypt message */
#define KSERR_SEQ -4			/* Incorrect sequence number */
#define KSERR_MALFORMED -5		/* Input ciphertext is broken */

/* --- Key exchange --- *
 *
 * TrIPE uses the Wrestlers Protocol for its key exchange.  The Wrestlers
 * Protocol has a number of desirable features (e.g., perfect forward
 * secrecy, and zero-knowledge authentication) which make it attractive for
 * use in TrIPE.  The Wrestlers Protocol was designed by Mark Wooding and
 * Clive Jones.
 */

typedef struct retry {
  double t;				/* Current retry time */
} retry;

#define KX_NCHAL 16u

typedef struct kxchal {
  struct keyexch *kx;			/* Pointer back to key exchange */
  ge *c;				/* Responder's challenge */
  ge *r;				/* My reply to the challenge */
  keyset *ks;				/* Pointer to temporary keyset */
  unsigned f;				/* Various useful flags */
  sel_timer t;				/* Response timer for challenge */
  retry rs;				/* Retry state */
  octet hc[MAXHASHSZ];			/* Hash of his challenge */
  octet ck[MAXHASHSZ];			/* His magical check value */
  octet hswrq_in[MAXHASHSZ];		/* Inbound switch request message */
  octet hswok_in[MAXHASHSZ];		/* Inbound switch confirmation */
  octet hswrq_out[MAXHASHSZ];		/* Outbound switch request message */
  octet hswok_out[MAXHASHSZ];		/* Outbound switch confirmation */
} kxchal;

typedef struct keyexch {
  struct peer *p;			/* Pointer back to the peer */
  kdata *kpriv;				/* Private key and related info */
  kdata *kpub;				/* Peer's public key */
  keyset **ks;				/* Peer's list of keysets */
  unsigned f;				/* Various useful flags */
  unsigned s;				/* Current state in exchange */
  sel_timer t;				/* Timer for next exchange */
  retry rs;				/* Retry state */
  mp *alpha;				/* My temporary secret */
  ge *c;				/* My challenge */
  ge *rx;				/* The expected response */
  unsigned nr;				/* Number of extant responses */
  time_t t_valid;			/* When this exchange goes bad */
  octet hc[MAXHASHSZ];			/* Hash of my challenge */
  kxchal *r[KX_NCHAL];			/* Array of challenges */
} keyexch;

#define KXF_TIMER 1u			/* Waiting for a timer to go off */
#define KXF_DEAD 2u			/* The key-exchanger isn't up */
#define KXF_PUBKEY 4u			/* Key exchanger has a public key */
#define KXF_CORK 8u			/* Don't send anything yet */

enum {
  KXS_DEAD,				/* Uninitialized state (magical) */
  KXS_CHAL,				/* Main answer-challenges state */
  KXS_COMMIT,				/* Committed: send switch request */
  KXS_SWITCH				/* Switched: send confirmation */
};

/* --- Tunnel structure --- *
 *
 * Used to maintain system-specific information about the tunnel interface.
 */

typedef struct tunnel tunnel;
struct peer;

typedef struct tunnel_ops {
  const char *name;			/* Name of this tunnel driver */
  unsigned flags;			/* Various interesting flags */
#define TUNF_PRIVOPEN 1u		/*   Need helper to open file */
  void (*init)(void);			/* Initializes the system */
  tunnel *(*create)(struct peer */*p*/, int /*fd*/, char **/*ifn*/);
					/* Initializes a new tunnel */
  void (*setifname)(tunnel */*t*/, const char */*ifn*/);
					/*  Notifies ifname change */
  void (*inject)(tunnel */*t*/, buf */*b*/); /* Sends packet through if */
  void (*destroy)(tunnel */*t*/);	/* Destroys a tunnel */
} tunnel_ops;

#ifndef TUN_INTERNALS
struct tunnel { const tunnel_ops *ops; };
#endif

/* --- Peer statistics --- *
 *
 * Contains various interesting and not-so-interesting statistics about a
 * peer.  This is updated by various parts of the code.  The format of the
 * structure isn't considered private, and @p_stats@ returns a pointer to the
 * statistics block for a given peer.
 */

typedef struct stats {
  unsigned long sz_in, sz_out;		/* Size of all data in and out */
  unsigned long sz_kxin, sz_kxout;	/* Size of key exchange messages */
  unsigned long sz_ipin, sz_ipout;	/* Size of encapsulated IP packets */
  time_t t_start, t_last, t_kx;		/* Time peer created, last pk, kx */
  unsigned long n_reject;		/* Number of rejected packets */
  unsigned long n_in, n_out;		/* Number of packets in and out */
  unsigned long n_kxin, n_kxout;	/* Number of key exchange packets */
  unsigned long n_ipin, n_ipout;	/* Number of encrypted packets */
} stats;

/* --- Peer structure --- *
 *
 * The main structure which glues everything else together.
 */

typedef struct peerspec {
  char *name;				/* Peer's name */
  char *privtag;			/* Private key tag */
  char *tag;				/* Public key tag */
  const tunnel_ops *tops;		/* Tunnel operations */
  unsigned long t_ka;			/* Keep alive interval */
  addr sa;				/* Socket address to speak to */
  size_t sasz;				/* Socket address size */
  unsigned f;				/* Flags for the peer */
#define PSF_KXMASK 255u			/*   Key-exchange flags to set */
#define PSF_MOBILE 256u			/*   Address may change rapidly */
} peerspec;

typedef struct peer_byname {
  sym_base _b;
  struct peer *p;
} peer_byname;

typedef struct peer_byaddr {
  addrmap_base _b;
  struct peer *p;
} peer_byaddr;

typedef struct peer {
  peer_byname *byname;			/* Lookup-by-name block */
  peer_byaddr *byaddr;			/* Lookup-by-address block */
  struct ping *pings;			/* Pings we're waiting for */
  peerspec spec;			/* Specifications for this peer */
  tunnel *t;				/* Tunnel for local packets */
  char *ifname;				/* Interface name for tunnel */
  keyset *ks;				/* List head for keysets */
  buf b;				/* Buffer for sending packets */
  stats st;				/* Statistics */
  keyexch kx;				/* Key exchange protocol block */
  sel_timer tka;			/* Timer for keepalives */
} peer;

typedef struct peer_iter { sym_iter i; } peer_iter;

typedef struct ping {
  struct ping *next, *prev;		/* Links to next and previous */
  peer *p;				/* Peer so we can free it */
  unsigned msg;				/* Kind of response expected */
  uint32 id;				/* Id so we can recognize response */
  octet magic[32];			/* Some random data */
  sel_timer t;				/* Timeout for ping */
  void (*func)(int /*rc*/, void */*arg*/); /* Function to call when done */
  void *arg;				/* Argument for callback */
} ping;

enum {
  PING_NONOTIFY = -1,
  PING_OK = 0,
  PING_TIMEOUT,
  PING_PEERDIED,
  PING_MAX
};

/* --- Admin structure --- */

#define OBUFSZ 16384u

typedef struct obuf {
  struct obuf *next;			/* Next buffer in list */
  char *p_in, *p_out;			/* Pointers into the buffer */
  char buf[OBUFSZ];			/* The actual buffer */
} obuf;

typedef struct oqueue {
  obuf *hd, *tl;			/* Head and tail pointers */
} oqueue;

struct admin;

typedef struct admin_bgop {
  struct admin_bgop *next, *prev;	/* Links to next and previous */
  struct admin *a;			/* Owner job */
  char *tag;				/* Tag string for messages */
  void (*cancel)(struct admin_bgop *);	/* Destructor function */
} admin_bgop;

typedef struct admin_resop {
  admin_bgop bg;			/* Background operation header */
  char *addr;				/* Hostname to be resolved */
  bres_client r;			/* Background resolver task */
  sel_timer t;				/* Timer for resolver */
  addr sa;				/* Socket address */
  size_t sasz;				/* Socket address size */
  void (*func)(struct admin_resop *, int); /* Handler */
} admin_resop;

enum { ARES_OK, ARES_FAIL };

typedef struct admin_addop {
  admin_resop r;			/* Name resolution header */
  peerspec peer;			/* Peer pending creation */
} admin_addop;

typedef struct admin_pingop {
  admin_bgop bg;			/* Background operation header */
  ping ping;				/* Ping pending response */
  struct timeval pingtime;		/* Time last ping was sent */
} admin_pingop;

typedef struct admin_service {
  sym_base _b;				/* Hash table base structure */
  char *version;			/* The provided version */
  struct admin *prov;			/* Which client provides me */
  struct admin_service *next, *prev;	/* Client's list of services */
} admin_service;

typedef struct admin_svcop {
  admin_bgop bg;			/* Background operation header */
  struct admin *prov;			/* Client servicing this job */
  unsigned index;			/* This job's index */
  struct admin_svcop *next, *prev;	/* Links for provider's jobs */
} admin_svcop;

typedef struct admin_jobentry {
  unsigned short seq;			/* Zero if unused */
  union {
    admin_svcop *op;			/* Operation, if slot in use, ... */
    uint32 next;			/* ... or index of next free slot */
  } u;
} admin_jobentry;

typedef struct admin_jobtable {
  uint32 n, sz;				/* Used slots and table size */
  admin_svcop *active;			/* List of active jobs */
  uint32 free;				/* Index of first free slot */
  admin_jobentry *v;			/* And the big array of entries */
} admin_jobtable;

typedef struct admin {
  struct admin *next, *prev;		/* Links to next and previous */
  unsigned f;				/* Various useful flags */
  unsigned ref;				/* Reference counter */
#ifndef NTRACE
  unsigned seq;				/* Sequence number for tracing */
#endif
  oqueue out;				/* Output buffer list */
  oqueue delay;				/* Delayed output buffer list */
  admin_bgop *bg;			/* Backgrounded operations */
  admin_service *svcs;			/* Which services I provide */
  admin_jobtable j;			/* Table of outstanding jobs */
  selbuf b;				/* Line buffer for commands */
  sel_file w;				/* Selector for write buffering */
} admin;

#define AF_DEAD 1u			/* Destroy this admin block */
#define AF_CLOSE 2u			/* Client closed connection */
#define AF_NOTE 4u			/* Catch notifications */
#define AF_WARN 8u			/* Catch warning messages */
#ifndef NTRACE
  #define AF_TRACE 16u			/* Catch tracing */
#endif
#define AF_FOREGROUND 32u		/* Quit server when client closes */

#ifndef NTRACE
#  define AF_ALLMSGS (AF_NOTE | AF_TRACE | AF_WARN)
#else
#  define AF_ALLMSGS (AF_NOTE | AF_WARN)
#endif

/*----- Global variables --------------------------------------------------*/

extern sel_state sel;			/* Global I/O event state */
extern octet buf_i[PKBUFSZ], buf_o[PKBUFSZ], buf_t[PKBUFSZ], buf_u[PKBUFSZ];
extern const tunnel_ops *tunnels[];	/* Table of tunnels (0-term) */
extern const tunnel_ops *tun_default;	/* Default tunnel to use */
extern kdata *master;			/* Default private key */
extern const char *tag_priv;		/* Default private key tag */

#ifndef NTRACE
extern const trace_opt tr_opts[];	/* Trace options array */
extern unsigned tr_flags;		/* Trace options flags */
#endif

/*----- Other macros ------------------------------------------------------*/

#define TIMER noise_timer(RAND_GLOBAL)

/*----- Key management ----------------------------------------------------*/

/* --- @km_init@ --- *
 *
 * Arguments:	@const char *privkr@ = private keyring file
 *		@const char *pubkr@ = public keyring file
 *		@const char *ptag@ = default private-key tag
 *
 * Returns:	---
 *
 * Use:		Initializes the key-management machinery, loading the
 *		keyrings and so on.
 */

extern void km_init(const char */*privkr*/, const char */*pubkr*/,
		    const char */*ptag*/);

/* --- @km_reload@ --- *
 *
 * Arguments:	---
 *
 * Returns:	Zero if OK, nonzero to force reloading of keys.
 *
 * Use:		Checks the keyrings to see if they need reloading.
 */

extern int km_reload(void);

/* --- @km_findpub@, @km_findpriv@ --- *
 *
 * Arguments:	@const char *tag@ = key tag to load
 *
 * Returns:	Pointer to the kdata object if successful, or null on error.
 *
 * Use:		Fetches a public or private key from the keyring.
 */

extern kdata *km_findpub(const char */*tag*/);
extern kdata *km_findpriv(const char */*tag*/);

/* --- @km_samealgsp@ --- *
 *
 * Arguments:	@const kdata *kdx, *kdy@ = two key data objects
 *
 * Returns:	Nonzero if their two algorithm selections are the same.
 *
 * Use:		Checks sameness of algorithm selections: used to ensure that
 *		peers are using sensible algorithms.
 */

extern int km_samealgsp(const kdata */*kdx*/, const kdata */*kdy*/);

/* --- @km_ref@ --- *
 *
 * Arguments:	@kdata *kd@ = pointer to the kdata object
 *
 * Returns:	---
 *
 * Use:		Claim a new reference to a kdata object.
 */

extern void km_ref(kdata */*kd*/);

/* --- @km_unref@ --- *
 *
 * Arguments:	@kdata *kd@ = pointer to the kdata object
 *
 * Returns:	---
 *
 * Use:		Releases a reference to a kdata object.
 */

extern void km_unref(kdata */*kd*/);

/* --- @km_tag@ --- *
 *
 * Arguments:	@kdata *kd@ - pointer to the kdata object
 *
 * Returns:	A pointer to the short tag by which the kdata was loaded.
 */

extern const char *km_tag(kdata */*kd*/);

/*----- Key exchange ------------------------------------------------------*/

/* --- @kx_start@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *		@int forcep@ = nonzero to ignore the quiet timer
 *
 * Returns:	---
 *
 * Use:		Stimulates a key exchange.  If a key exchage is in progress,
 *		a new challenge is sent (unless the quiet timer forbids
 *		this); if no exchange is in progress, one is commenced.
 */

extern void kx_start(keyexch */*kx*/, int /*forcep*/);

/* --- @kx_message@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *		@unsigned msg@ = the message code
 *		@buf *b@ = pointer to buffer containing the packet
 *
 * Returns:	---
 *
 * Use:		Reads a packet containing key exchange messages and handles
 *		it.
 */

extern void kx_message(keyexch */*kx*/, unsigned /*msg*/, buf */*b*/);

/* --- @kx_free@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *
 * Returns:	---
 *
 * Use:		Frees everything in a key exchange context.
 */

extern void kx_free(keyexch */*kx*/);

/* --- @kx_newkeys@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *
 * Returns:	---
 *
 * Use:		Informs the key exchange module that its keys may have
 *		changed.  If fetching the new keys fails, the peer will be
 *		destroyed, we log messages and struggle along with the old
 *		keys.
 */

extern void kx_newkeys(keyexch */*kx*/);

/* --- @kx_init@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *		@peer *p@ = pointer to peer context
 *		@keyset **ks@ = pointer to keyset list
 *		@unsigned f@ = various useful flags
 *
 * Returns:	Zero if OK, nonzero if it failed.
 *
 * Use:		Initializes a key exchange module.  The module currently
 *		contains no keys, and will attempt to initiate a key
 *		exchange.
 */

extern int kx_init(keyexch */*kx*/, peer */*p*/,
		   keyset **/*ks*/, unsigned /*f*/);

/*----- Keysets and symmetric cryptography --------------------------------*/

/* --- @ks_drop@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Decrements a keyset's reference counter.  If the counter hits
 *		zero, the keyset is freed.
 */

extern void ks_drop(keyset */*ks*/);

/* --- @ks_gen@ --- *
 *
 * Arguments:	@const void *k@ = pointer to key material
 *		@size_t x, y, z@ = offsets into key material (see below)
 *		@peer *p@ = pointer to peer information
 *
 * Returns:	A pointer to the new keyset.
 *
 * Use:		Derives a new keyset from the given key material.  The
 *		offsets @x@, @y@ and @z@ separate the key material into three
 *		parts.  Between the @k@ and @k + x@ is `my' contribution to
 *		the key material; between @k + x@ and @k + y@ is `your'
 *		contribution; and between @k + y@ and @k + z@ is a shared
 *		value we made together.  These are used to construct two
 *		pairs of symmetric keys.  Each pair consists of an encryption
 *		key and a message authentication key.  One pair is used for
 *		outgoing messages, the other for incoming messages.
 *
 *		The new key is marked so that it won't be selected for output
 *		by @ksl_encrypt@.  You can still encrypt data with it by
 *		calling @ks_encrypt@ directly.
 */

extern keyset *ks_gen(const void */*k*/,
		      size_t /*x*/, size_t /*y*/, size_t /*z*/,
		      peer */*p*/);

/* --- @ks_activate@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Activates a keyset, so that it can be used for encrypting
 *		outgoing messages.
 */

extern void ks_activate(keyset */*ks*/);

/* --- @ks_encrypt@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *		@unsigned ty@ = message type
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Zero if successful; @KSERR_REGEN@ if we should negotiate a
 *		new key; @KSERR_NOKEYS@ if the key is not usable.  Also
 *		returns zero if there was insufficient buffer (but the output
 *		buffer is broken in this case).
 *
 * Use:		Encrypts a block of data using the key.  Note that the `key
 *		ought to be replaced' notification is only ever given once
 *		for each key.  Also note that this call forces a keyset to be
 *		used even if it's marked as not for data output.
 */

extern int ks_encrypt(keyset */*ks*/, unsigned /*ty*/,
		      buf */*b*/, buf */*bb*/);

/* --- @ks_decrypt@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *		@unsigned ty@ = expected type code
 *		@buf *b@ = pointer to an input buffer
 *		@buf *bb@ = pointer to an output buffer
 *
 * Returns:	Zero on success; @KSERR_DECRYPT@ on failure.  Also returns
 *		zero if there was insufficient buffer (but the output buffer
 *		is broken in this case).
 *
 * Use:		Attempts to decrypt a message using a given key.  Note that
 *		requesting decryption with a key directly won't clear a
 *		marking that it's not for encryption.
 */

extern int ks_decrypt(keyset */*ks*/, unsigned /*ty*/,
		      buf */*b*/, buf */*bb*/);

/* --- @ksl_free@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *
 * Returns:	---
 *
 * Use:		Frees (releases references to) all of the keys in a keyset.
 */

extern void ksl_free(keyset **/*ksroot*/);

/* --- @ksl_link@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Links a keyset into a list.  A keyset can only be on one list
 *		at a time.  Bad things happen otherwise.
 */

extern void ksl_link(keyset **/*ksroot*/, keyset */*ks*/);

/* --- @ksl_prune@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *
 * Returns:	---
 *
 * Use:		Prunes the keyset list by removing keys which mustn't be used
 *		any more.
 */

extern void ksl_prune(keyset **/*ksroot*/);

/* --- @ksl_encrypt@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@unsigned ty@ = message type
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Zero if successful; @KSERR_REGEN@ if it's time to negotiate a
 *		new key; @KSERR_NOKEYS@ if there are no suitable keys
 *		available.  Also returns zero if there was insufficient
 *		buffer space (but the output buffer is broken in this case).
 *
 * Use:		Encrypts a packet.
 */

extern int ksl_encrypt(keyset **/*ksroot*/, unsigned /*ty*/,
		       buf */*b*/, buf */*bb*/);

/* --- @ksl_decrypt@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@unsigned ty@ = expected type code
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Zero on success; @KSERR_DECRYPT@ on failure.  Also returns
 *		zero if there was insufficient buffer (but the output buffer
 *		is broken in this case).
 *
 * Use:		Decrypts a packet.
 */

extern int ksl_decrypt(keyset **/*ksroot*/, unsigned /*ty*/,
		       buf */*b*/, buf */*bb*/);

/*----- Challenges --------------------------------------------------------*/

/* --- @c_new@ --- *
 *
 * Arguments:	@buf *b@ = where to put the challenge
 *
 * Returns:	Zero if OK, nonzero on error.
 *
 * Use:		Issues a new challenge.
 */

extern int c_new(buf */*b*/);

/* --- @c_check@ --- *
 *
 * Arguments:	@buf *b@ = where to find the challenge
 *
 * Returns:	Zero if OK, nonzero if it didn't work.
 *
 * Use:		Checks a challenge.  On failure, the buffer is broken.
 */

extern int c_check(buf */*b*/);

/*----- Administration interface ------------------------------------------*/

#define A_END ((char *)0)

/* --- @a_vformat@ --- *
 *
 * Arguments:	@dstr *d@ = where to leave the formatted message
 *		@const char *fmt@ = pointer to format string
 *		@va_list ap@ = arguments in list
 *
 * Returns:	---
 *
 * Use:		Main message token formatting driver.  The arguments are
 *		interleaved formatting tokens and their parameters, finally
 *		terminated by an entry @A_END@.
 *
 *		Tokens recognized:
 *
 *		  * "*..." ... -- pretokenized @dstr_putf@-like string
 *
 *		  * "?ADDR" SOCKADDR -- a socket address, to be converted
 *
 *		  * "?B64" BUFFER SIZE -- binary data to be base64-encoded
 *
 *		  * "?TOKENS" VECTOR -- null-terminated vector of tokens
 *
 *		  * "?PEER" PEER -- peer's name
 *
 *		  * "?ERRNO" ERRNO -- system error code
 *
 *		  * "[!]..." ... -- @dstr_putf@-like string as single token
 */

extern void a_vformat(dstr */*d*/, const char */*fmt*/, va_list /*ap*/);

/* --- @a_format@ --- *
 *
 * Arguments:	@dstr *d@ = where to leave the formatted message
 *		@const char *fmt@ = pointer to format string
 *
 * Returns:	---
 *
 * Use:		Writes a tokenized message into a string, for later
 *		presentation.
 */

extern void a_format(dstr */*d*/, const char */*fmt*/, ...);

/* --- @a_warn@ --- *
 *
 * Arguments:	@const char *fmt@ = pointer to format string
 *		@...@ = other arguments
 *
 * Returns:	---
 *
 * Use:		Informs all admin connections of a warning.
 */

extern void a_warn(const char */*fmt*/, ...);

/* --- @a_notify@ --- *
 *
 * Arguments:	@const char *fmt@ = pointer to format string
 *		@...@ = other arguments
 *
 * Returns:	---
 *
 * Use:		Sends a notification to interested admin connections.
 */

extern void a_notify(const char */*fmt*/, ...);

/* --- @a_create@ --- *
 *
 * Arguments:	@int fd_in, fd_out@ = file descriptors to use
 *		@unsigned f@ = initial flags to set
 *
 * Returns:	---
 *
 * Use:		Creates a new admin connection.
 */

extern void a_create(int /*fd_in*/, int /*fd_out*/, unsigned /*f*/);

/* --- @a_quit@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Shuts things down nicely.
 */

extern void a_quit(void);

/* --- @a_preselect@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Informs the admin module that we're about to select again,
 *		and that it should do cleanup things it has delayed until a
 *		`safe' time.
 */

extern void a_preselect(void);

/* --- @a_daemon@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Informs the admin module that it's a daemon.
 */

extern void a_daemon(void);

/* --- @a_init@ --- *
 *
 * Arguments:	@const char *sock@ = socket name to create
 *		@uid_t u@ = user to own the socket
 *		@gid_t g@ = group to own the socket
 *		@mode_t m@ = permissions to set on the socket
 *
 * Returns:	---
 *
 * Use:		Creates the admin listening socket.
 */

extern void a_init(const char */*sock*/,
		   uid_t /*u*/, gid_t /*g*/, mode_t /*m*/);

/*----- Mapping with addresses as keys ------------------------------------*/

/* --- @am_create@ --- *
 *
 * Arguments:	@addrmap *m@ = pointer to map
 *
 * Returns:	---
 *
 * Use:		Create an address map, properly set up.
 */

extern void am_create(addrmap */*m*/);

/* --- @am_destroy@ --- *
 *
 * Arguments:	@addrmap *m@ = pointer to map
 *
 * Returns:	---
 *
 * Use:		Destroy an address map, throwing away all the entries.
 */

extern void am_destroy(addrmap */*m*/);

/* --- @am_find@ --- *
 *
 * Arguments:	@addrmap *m@ = pointer to map
 *		@const addr *a@ = address to look up
 *		@size_t sz@ = size of block to allocate
 *		@unsigned *f@ = where to store flags
 *
 * Returns:	Pointer to found item, or null.
 *
 * Use:		Finds a record with the given IP address, set @*f@ nonzero
 *		and returns it.  If @sz@ is zero, and no match was found,
 *		return null; otherwise allocate a new block of @sz@ bytes,
 *		clear @*f@ to zero and return the block pointer.
 */

extern void *am_find(addrmap */*m*/, const addr */*a*/,
		     size_t /*sz*/, unsigned */*f*/);

/* --- @am_remove@ --- *
 *
 * Arguments:	@addrmap *m@ = pointer to map
 *		@void *i@ = pointer to the item
 *
 * Returns:	---
 *
 * Use:		Removes an item from the map.
 */

extern void am_remove(addrmap */*m*/, void */*i*/);

/*----- Privilege separation ----------------------------------------------*/

/* --- @ps_trace@ --- *
 *
 * Arguments:	@unsigned mask@ = trace mask to check
 *		@const char *fmt@ = message format
 *		@...@ = values for placeholders
 *
 * Returns:	---
 *
 * Use:		Writes a trace message.
 */

T( extern void ps_trace(unsigned /*mask*/, const char */*fmt*/, ...); )

/* --- @ps_warn@ --- *
 *
 * Arguments:	@const char *fmt@ = message format
 *		@...@ = values for placeholders
 *
 * Returns:	---
 *
 * Use:		Writes a warning message.
 */

extern void ps_warn(const char */*fmt*/, ...);

/* --- @ps_tunfd@ --- *
 *
 * Arguments:	@const tunnel_ops *tops@ = pointer to tunnel operations
 *		@char **ifn@ = where to put the interface name
 *
 * Returns:	The file descriptor, or @-1@ on error.
 *
 * Use:		Fetches a file descriptor for a tunnel driver.
 */

extern int ps_tunfd(const tunnel_ops */*tops*/, char **/*ifn*/);

/* --- @ps_split@ --- *
 *
 * Arguments:	@int detachp@ = whether to detach the child from its terminal
 *
 * Returns:	---
 *
 * Use:		Separates off the privileged tunnel-opening service from the
 *		rest of the server.
 */

extern void ps_split(int /*detachp*/);

/* --- @ps_quit@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Detaches from the helper process.
 */

extern void ps_quit(void);

/*----- Peer management ---------------------------------------------------*/

/* --- @p_txstart@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *		@unsigned msg@ = message type code
 *
 * Returns:	A pointer to a buffer to write to.
 *
 * Use:		Starts sending to a peer.  Only one send can happen at a
 *		time.
 */

extern buf *p_txstart(peer */*p*/, unsigned /*msg*/);

/* --- @p_txend@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *
 * Returns:	---
 *
 * Use:		Sends a packet to the peer.
 */

extern void p_txend(peer */*p*/);

/* --- @p_pingsend@ --- *
 *
 * Arguments:	@peer *p@ = destination peer
 *		@ping *pg@ = structure to fill in
 *		@unsigned type@ = message type
 *		@unsigned long timeout@ = how long to wait before giving up
 *		@void (*func)(int, void *)@ = callback function
 *		@void *arg@ = argument for callback
 *
 * Returns:	Zero if successful, nonzero if it failed.
 *
 * Use:		Sends a ping to a peer.  Call @func@ with a nonzero argument
 *		if we get an answer within the timeout, or zero if no answer.
 */

extern int p_pingsend(peer */*p*/, ping */*pg*/, unsigned /*type*/,
		      unsigned long /*timeout*/,
		      void (*/*func*/)(int, void *), void */*arg*/);

/* --- @p_pingdone@ --- *
 *
 * Arguments:	@ping *p@ = ping structure
 *		@int rc@ = return code to pass on
 *
 * Returns:	---
 *
 * Use:		Disposes of a ping structure, maybe sending a notification.
 */

extern void p_pingdone(ping */*p*/, int /*rc*/);

/* --- @p_greet@ --- *
 *
 * Arguments:	@peer *p@ = peer to send to
 *		@const void *c@ = pointer to challenge
 *		@size_t sz@ = size of challenge
 *
 * Returns:	---
 *
 * Use:		Sends a greeting packet.
 */

extern void p_greet(peer */*p*/, const void */*c*/, size_t /*sz*/);

/* --- @p_tun@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *		@buf *b@ = buffer containing incoming packet
 *
 * Returns:	---
 *
 * Use:		Handles a packet which needs to be sent to a peer.
 */

extern void p_tun(peer */*p*/, buf */*b*/);

/* --- @p_keyreload@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Forces a check of the daemon's keyring files.
 */

extern void p_keyreload(void);

/* --- @p_interval@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Called periodically to do tidying.
 */

extern void p_interval(void);

/* --- @p_stats@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's statistics.
 */

extern stats *p_stats(peer */*p*/);

/* --- @p_ifname@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's interface name.
 */

extern const char *p_ifname(peer */*p*/);

/* --- @p_setifname@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *		@const char *name@ = pointer to the new name
 *
 * Returns:	---
 *
 * Use:		Changes the name held for a peer's interface.
 */

extern void p_setifname(peer */*p*/, const char */*name*/);

/* --- @p_addr@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's address.
 */

extern const addr *p_addr(peer */*p*/);

/* --- @p_init@ --- *
 *
 * Arguments:	@struct in_addr addr@ = address to bind to
 *		@unsigned port@ = port number to listen to
 *
 * Returns:	---
 *
 * Use:		Initializes the peer system; creates the socket.
 */

extern void p_init(struct in_addr /*addr*/, unsigned /*port*/);

/* --- @p_port@ --- *
 *
 * Arguments:	---
 *
 * Returns:	Port number used for socket.
 */

unsigned p_port(void);

/* --- @p_create@ --- *
 *
 * Arguments:	@peerspec *spec@ = information about this peer
 *
 * Returns:	Pointer to the peer block, or null if it failed.
 *
 * Use:		Creates a new named peer block.  No peer is actually attached
 *		by this point.
 */

extern peer *p_create(peerspec */*spec*/);

/* --- @p_name@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's name.
 *
 * Use:		Equivalent to @p_spec(p)->name@.
 */

extern const char *p_name(peer */*p*/);

/* --- @p_tag@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's public key tag.
 */

extern const char *p_tag(peer */*p*/);

/* --- @p_privtag@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's private key tag.
 */

extern const char *p_privtag(peer */*p*/);

/* --- @p_spec@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	Pointer to the peer's specification
 */

extern const peerspec *p_spec(peer */*p*/);

/* --- @p_findbyaddr@ --- *
 *
 * Arguments:	@const addr *a@ = address to look up
 *
 * Returns:	Pointer to the peer block, or null if not found.
 *
 * Use:		Finds a peer by address.
 */

extern peer *p_findbyaddr(const addr */*a*/);

/* --- @p_find@ --- *
 *
 * Arguments:	@const char *name@ = name to look up
 *
 * Returns:	Pointer to the peer block, or null if not found.
 *
 * Use:		Finds a peer by name.
 */

extern peer *p_find(const char */*name*/);

/* --- @p_destroy@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer
 *
 * Returns:	---
 *
 * Use:		Destroys a peer.
 */

extern void p_destroy(peer */*p*/);

/* --- @FOREACH_PEER@ --- *
 *
 * Arguments:	@p@ = name to bind to each peer
 *		@stuff@ = thing to do for each item
 *
 * Use:		Does something for each current peer.
 */

#define FOREACH_PEER(p, stuff) do {					\
  peer_iter i_;								\
  peer *p;								\
  for (p_mkiter(&i_); (p = p_next(&i_)) != 0; ) stuff			\
} while (0)

/* --- @p_mkiter@ --- *
 *
 * Arguments:	@peer_iter *i@ = pointer to an iterator
 *
 * Returns:	---
 *
 * Use:		Initializes the iterator.
 */

extern void p_mkiter(peer_iter */*i*/);

/* --- @p_next@ --- *
 *
 * Arguments:	@peer_iter *i@ = pointer to an iterator
 *
 * Returns:	Next peer, or null if at the end.
 *
 * Use:		Returns the next peer.
 */

extern peer *p_next(peer_iter */*i*/);

/*----- Tunnel drivers ----------------------------------------------------*/

#ifdef TUN_LINUX
  extern const tunnel_ops tun_linux;
#endif

#ifdef TUN_UNET
  extern const tunnel_ops tun_unet;
#endif

#ifdef TUN_BSD
  extern const tunnel_ops tun_bsd;
#endif

extern const tunnel_ops tun_slip;

/*----- Other handy utilities ---------------------------------------------*/

/* --- @mpstr@ --- *
 *
 * Arguments:	@mp *m@ = a multiprecision integer
 *
 * Returns:	A pointer to the integer's textual representation.
 *
 * Use:		Converts a multiprecision integer to a string.  Corrupts
 *		@buf_u@.
 */

extern const char *mpstr(mp */*m*/);

/* --- @gestr@ --- *
 *
 * Arguments:	@group *g@ = a group
 *		@ge *x@ = a group element
 *
 * Returns:	A pointer to the element's textual representation.
 *
 * Use:		Converts a group element to a string.  Corrupts
 *		@buf_u@.
 */

extern const char *gestr(group */*g*/, ge */*x*/);

/* --- @timestr@ --- *
 *
 * Arguments:	@time_t t@ = a time to convert
 *
 * Returns:	A pointer to a textual representation of the time.
 *
 * Use:		Converts a time to a textual representation.  Corrupts
 *		@buf_u@.
 */

extern const char *timestr(time_t /*t*/);

/* --- @mystrieq@ --- *
 *
 * Arguments:	@const char *x, *y@ = two strings
 *
 * Returns:	True if @x@ and @y are equal, up to case.
 */

extern int mystrieq(const char */*x*/, const char */*y*/);

/* --- @seq_reset@ --- *
 *
 * Arguments:	@seqwin *s@ = sequence-checking window
 *
 * Returns:	---
 *
 * Use:		Resets a sequence number window.
 */

extern void seq_reset(seqwin */*s*/);

/* --- @seq_check@ --- *
 *
 * Arguments:	@seqwin *s@ = sequence-checking window
 *		@uint32 q@ = sequence number to check
 *		@const char *service@ = service to report message from
 *
 * Returns:	A @SEQ_@ code.
 *
 * Use:		Checks a sequence number against the window, updating things
 *		as necessary.
 */

extern int seq_check(seqwin */*s*/, uint32 /*q*/, const char */*service*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
